/*
Comments:
Slice an object along any arbitrary plane.
*/

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;
float4x4 ViewInvXf : ViewInverse < string UIWidget="None"; >;


float3 Lamp0Pos : Position <
    string Object = "PointLight0";
    string UIName =  "Lamp 0 Position";
    string Space = "World";
> = {-0.5f,2.0f,1.25f};

float4 LiteColor: COLOR <
    string UIName =  "Bright Surface";
    string UIWidget = "Color";
> = {0.8f, 0.5f, 0.1f, 1};

float4 DarkColor: COLOR <
    string UIName =  "Dark Surface";
    string UIWidget = "Color";
> = {0.0f, 0.0f, 0.0f, 1};

float4 WarmColor: COLOR <
    string UIName =  "Gooch Warm Tone";
    string UIWidget = "Color";
> = {0.5f, 0.4f, 0.05f, 1};

float4 CoolColor: COLOR <
    string UIName =  "Gooch Cool Tone";
    string UIWidget = "Color";
> = {0.05f, 0.05f, 0.6f, 1};

float4 SpecColor: COLOR <
    string UIName =  "Highlight";
    string UIWidget = "Color";
> = {0.7f, 0.7f, 1.0f, 1};

float SpecExpon : SpecularPower <
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName =  "Specular Power";
> = 40.0;

float4x4 SlicerXf : Here <
	string UIName = "Slicer Transform";
	string Object = "SpotLight0";
	string frustum = "light0";
>;

//////////////////////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

/* data passed from vertex shader to pixel shader */
struct sliceVertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord	: TEXCOORD0;
    float3 LightVec	: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldPos	: TEXCOORD3;
    float3 WorldView	: TEXCOORD4;
    float3 SlicePos	: TEXCOORD5;
};

/*********** vertex shader ******/

sliceVertexOutput mainVS(appdata IN)
{
    sliceVertexOutput OUT = (sliceVertexOutput)0;
    OUT.WorldNormal = mul(IN.Normal,WorldITXf).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po,WorldXf).xyz;
    float3 Ps = mul(Po,SlicerXf).xyz;
    OUT.WorldPos = Pw;
    OUT.SlicePos = Ps;
    OUT.LightVec = Lamp0Pos - Pw;
    OUT.TexCoord = IN.UV;
    OUT.WorldView = ViewInvXf[3].xyz - Pw;
    OUT.HPosition = mul(Po,WvpXf);
    return OUT;
}

/*********** pixel shader ******/

float4 slicerPS(sliceVertexOutput IN) :COLOR
{
    float3 Ln = normalize(IN.LightVec);
    float3 Vn = normalize(IN.WorldView);
    float3 Nn = normalize(IN.WorldNormal);
    Nn = faceforward(Nn,-Vn,Nn);
    float3 Hn = normalize(Vn + Ln);
    float hdn = pow(max(0,dot(Hn,Nn)),SpecExpon);
    float4 specContrib = float4((hdn * SpecColor.xyz),1);
    float ldn = dot(Ln,Nn);
    float mixer = 0.5 * (ldn + 1.0);
    float diffComp = max(0,ldn);
    float3 surfColor = lerp(DarkColor,LiteColor,mixer);
    float3 toneColor = lerp(CoolColor,WarmColor,mixer);
    float4 diffContrib = float4((surfColor + toneColor),1);
    float4 result = diffContrib + specContrib;
	clip(IN.SlicePos.zzz);
    return result;
}



technique Main {
    pass p0 {
        VertexShader = compile vs_2_0 mainVS();/*
	    ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;*/
        PixelShader = compile ps_2_a slicerPS();
    }
}
