#region usings
using System;
using System.ComponentModel.Composition;
using System.Runtime.InteropServices;
using System.Collections;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectInput;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "DirectInput", Category = "Devices", Help = "Access mouse data using DirectInput", Tags = "")]
	#endregion PluginInfo
	public class DevicesDirectInputNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Window Handle", DefaultValue = -1, IsSingle = true)]
		ISpread<int> FHandle;
		[Input("Foreground", DefaultValue = 0, IsSingle = true)]
		ISpread<bool> FFrg;
		[Input("Exclusive", DefaultValue = 0, IsSingle = true)]
		ISpread<bool> FExclusive;
		[Input("Reinitialize", DefaultValue = 0, IsSingle = true)]
		ISpread<bool> FInit;

		[Output("Mouse Position XYW")]
		ISpread<int> FPos;
		[Output("Mouse Buttons")]
		ISpread<bool> FButton;
		[Output("Tab")]
		ISpread<bool> FKeys;

		[Import()]
		ILogger FLogger;
		#endregion fields & pins
		
		[DllImport("C:\\Windows\\System32\\user32.dll")]
		public static extern IntPtr GetForegroundWindow();
		
		private Device mouse;
		private Device keyb;
		
		private void InitDevice() {
			mouse = new Device(SystemGuid.Mouse);
			keyb = new Device(SystemGuid.Keyboard);
			if (mouse == null) FLogger.Log(LogType.Debug, "No mouse found.");
			if (keyb == null) FLogger.Log(LogType.Debug, "No keyboard found.");
			IntPtr handle = (FHandle[0]==-1) ? GetForegroundWindow() : new IntPtr(FHandle[0]);
			if(FFrg[0] && (!FExclusive[0])) {
				mouse.SetCooperativeLevel(
					handle,
					CooperativeLevelFlags.Foreground | CooperativeLevelFlags.NonExclusive
				);
				keyb.SetCooperativeLevel(
					handle,
					CooperativeLevelFlags.Foreground | CooperativeLevelFlags.NonExclusive
				);
			}
			if(FFrg[0] && FExclusive[0]) {
				mouse.SetCooperativeLevel(
					handle,
					CooperativeLevelFlags.Foreground | CooperativeLevelFlags.Exclusive
				);
				keyb.SetCooperativeLevel(
					handle,
					CooperativeLevelFlags.Foreground | CooperativeLevelFlags.Exclusive
				);
			}
			if(!FFrg[0]) {
				mouse.SetCooperativeLevel(
					handle,
					CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive
				);
				keyb.SetCooperativeLevel(
					handle,
					CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive
				);
			}
			mouse.Acquire();
			keyb.Acquire();
		}
		
		[ImportingConstructor]
		public DevicesDirectInputNode() {
			mouse = new Device(SystemGuid.Mouse);
			mouse.Properties.AxisModeAbsolute = false;
			mouse.Acquire();
			keyb = new Device(SystemGuid.Keyboard);
			keyb.Acquire();
		}
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			if(FInit[0]) InitDevice();
			if(mouse == null) FLogger.Log(LogType.Debug, "No mouse found.");
			MouseState state = mouse.CurrentMouseState;
			KeyboardState kstate = keyb.GetCurrentKeyboardState();
			FPos.SliceCount = 3;
			FPos[0] = state.X;
			FPos[1] = state.Y;
			FPos[2] = state.Z;
			
			byte[] buttons = state.GetMouseButtons();
			FButton.SliceCount = buttons.Length;
			for(int i=0; i < buttons.Length; i++) {
				FButton[i] = buttons[i]!=0;
			}
			FKeys[0] = kstate[Key.Tab];
		}
	}
}